import logging
from pathlib import Path

import yaml

from defence360agent.utils import antivirus_mode, atomic_rewrite

logger = logging.getLogger(__name__)

_NGINX_CONFIG = Path("/etc/imunify360-webshield/invisible-captcha.conf")
_NGINX_CONFIG_TPL = """
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#   DO NOT EDIT. AUTOMATICALLY GENERATED.
#   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#
#   Direct modifications to this file WILL be lost upon subsequent
#   regeneration of this configuration file.
#
#   To have your modifications retained, you should use CLI command
#     imunify360-agent features <install|remove> <feature>
#   or activate/deactivate appropriate feature in UI.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

set $invisible_captcha {invisible_captcha_on_off};
"""

_LOCAL_CONFIG = Path("/etc/sysconfig/imunify360/imunify360.config")


@antivirus_mode.skip
def migrate(
    migrator,
    database,
    fake=False,
    local_config=_LOCAL_CONFIG,
    nginx_config=_NGINX_CONFIG,
    **kwargs,
):
    def get_invisible_captcha():
        try:
            local_config_text = local_config.read_text()
        except FileNotFoundError:
            logger.warning("Imunify360 config doesn't exist")
            return False
        try:
            config = yaml.safe_load(local_config_text)
        except yaml.YAMLError:
            logger.warning("Imunify360 config is not valid YAML document")
            return False
        try:
            return config["WEBSHIELD"]["invisible_captcha"]
        except KeyError:
            return False

    def set_invisible_captcha(new_state: str):
        new_conf = _NGINX_CONFIG_TPL.format(invisible_captcha_on_off=new_state)
        try:
            atomic_rewrite(nginx_config, new_conf, backup=False)
        except Exception:
            logger.exception(f"Error writing {nginx_config.name}")

    if nginx_config.exists():
        logger.warning(f"{nginx_config.name} exists, skipping...")
        return

    state = "on" if get_invisible_captcha() else "off"
    set_invisible_captcha(state)


@antivirus_mode.skip
def rollback(migrator, database, fake=False, **kwargs):
    pass
